// WinRegistry.h: interface for the CRegistryKey and CRegistryValue classes.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_WINREGISTRY_H__82DB35FD_FDCD_4B7C_B9D1_8AA4CDE4530E__INCLUDED_)
#define AFX_WINREGISTRY_H__82DB35FD_FDCD_4B7C_B9D1_8AA4CDE4530E__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <string>
#include <windows.h>
#include "PreferenceValue.h"

class CRegistryValue;

class CRegistryKey
{
public:
	class CRegKeyInfo
	{
	public:
		CRegKeyInfo();
		CRegKeyInfo(HKEY hKey);
		~CRegKeyInfo();

		const CRegKeyInfo & operator = ( const CRegKeyInfo & refToCopy);

		std::string m_strClass;
		DWORD	m_dwSubKeys;				//  buffer for number of subkeys
		DWORD	m_dwMaxSubKeyLen;			//  buffer for longest subkey name length
		DWORD	m_dwMaxClassLen;			//  buffer for longest class string length
		DWORD	m_dwValues;					//  buffer for number of value entries
		DWORD	m_dwMaxValueNameLen;		//  buffer for longest value name length
		DWORD	m_dwMaxValueLen;			//  buffer for longest value data length
		DWORD	m_dwSecurityDescriptor;		//  buffer for security descriptor length
		FILETIME m_ftLastWriteTime;			//  buffer for last write time

	};

	struct enum_context
	{
		DWORD		dwCurrent;
		char		*pszNameBuffer;
		DWORD		nBufferLen;
	};

	CRegistryKey();
	virtual ~CRegistryKey();

	bool Open(const std::string & strPath, DWORD dwAccess = KEY_ALL_ACCESS);
	void Close();
	bool Create( const std::string & strPath, DWORD dwAccess = KEY_ALL_ACCESS, DWORD dwOptions = REG_OPTION_NON_VOLATILE );
	bool Destroy();

	bool GetSubKey( CRegistryKey & refKey, const std::string & strSubKey, DWORD dwAccess = KEY_ALL_ACCESS );
	bool EnumSubKeys( CRegistryKey & refKey, enum_context *& pContext, DWORD dwAccess = KEY_ALL_ACCESS ) const;
	bool EnumValues( CRegistryValue & refValue, std::string & strName, enum_context *& pContext ) const;
	void CreateEnumContext( enum_context *& pContext ) const;
	void DestroyEnumContext( enum_context *& pContext ) const;
	const std::string  & Name(void) {return(m_strKeyName);}
	operator HKEY () const {return(m_hKey);}
	const CRegistryKey & operator = (const CRegistryKey & refKeyToCopy);
	const CRegKeyInfo & GetKeyInfo() const {return(m_theKeyInfo);}

protected:
	bool Create(HKEY hKey, const std::string & strSubKey, DWORD dwAccess, DWORD dwOptions);
	bool Open(HKEY hKey, const std::string & strSubKey, DWORD dwAccess);
	void ExtractKeyHandleAndSubKey(HKEY & hKey, std::string & strSubKey, const std::string & strPath) const;
	void ExtractKeyName(std::string & m_strKeyName, const std::string & strPath) const;
	
	HKEY		m_hKey;
	std::string	m_strKeyName;
	std::string	m_strPath;
	CRegKeyInfo	m_theKeyInfo;
};

class CRegistryValue
{
public:
	CRegistryValue(){}
	virtual ~CRegistryValue(){}

	bool Open(CRegistryKey & refKey, const std::string & strName);
	bool Create(CRegistryKey & refKey, const std::string & strName, const PreferenceStore::SPreferenceValue & refValue);
	bool EnumValue(const CRegistryKey & refKey, size_t nIndex, CRegistryKey::enum_context * pContext = NULL);
	bool Destroy(void);

	void GetValue( PreferenceStore::SPreferenceValue & refValue ) {refValue = m_theValue;}
	bool SetValue(const PreferenceStore::SPreferenceValue & refValue);
	const std::string & GetName(void) {return(m_strName);}

protected:
	void ConvertToValue( PreferenceStore::SPreferenceValue & refValue, const std::string & strData ) const;
	void Trim( std::string & strToTrim, const std::string & strRemove = " " ) const;

	DECLARE_PREFIX_TABLE();
	DECLARE_POSTFIX_TABLE();

private:
	CRegistryKey m_theRegKey;
	std::string m_strName;
	PreferenceStore::SPreferenceValue m_theValue;
};

#endif	//	!defined(AFX_WINREGISTRY_H__82DB35FD_FDCD_4B7C_B9D1_8AA4CDE4530E__INCLUDED_)